import java.util.ArrayList;
import java.util.List;
/**
 *  Przykad wykorzystania agenta oraz instrumentacji
 *  
 *  W katalogu, w ktrym ze skompilowanymi klasami (bin):
 *  1. utwrz jar
 *  > jar cmf MANIFEST.MF InstrumentationAgent.jar InstrumentationAgent.class
 *  2. odpal aplikacj z agentem:
 *  > java -javaagent:InstrumentationAgent.jar InstrumentationExample
 *  
 *  Materia zrdowy:
 *  https://www.baeldung.com/java-size-of-object
 *  
 *  
 *  Java instrumentation API:
 *  https://docs.oracle.com/en/java/javase/11/docs/api/java.instrument/java/lang/instrument/Instrumentation.html
 *  
 *  Bardziej zaawansowany przykad instrumentacji:
 *   https://www.baeldung.com/java-instrumentation
 *  
 *  Biblioteka Javaassist:
 *  http://www.javassist.org/
 *  http://www.javassist.org/tutorial/tutorial.html
 *  https://www.baeldung.com/javassist
 *  
 *  Biblioteka cglib (proxy):
 *  https://www.baeldung.com/cglib
 *  IMPORTANT NOTE: cglib is unmaintained and does not work well (or possibly at all?) in newer JDKs, particularly JDK17+. If you need to support newer JDKs, we will accept well-tested well-thought-out patches... but you'll probably have better luck migrating to something like ByteBuddy.
 *  
 *  zastpca: https://bytebuddy.net/
 *  https://bytebuddy.net/#/tutorial
 *  
 *  Java Virtual Machine Tool Interface (JVM TI):
 *  https://docs.oracle.com/javase/8/docs/technotes/guides/jvmti/index.html (stare)
 *  https://docs.oracle.com/en/java/javase/11/docs/specs/jvmti.html (nowe, z moduami)
 *  - mona wykorzysta do profilowania
 *  
 *  https://www.baeldung.com/java-profilers
 *  
 *  
 *  OSGI:
 *  https://www.vogella.com/tutorials/OSGi/article.html
 *  https://www.baeldung.com/osgi
 *  
 *  Implementacja agenta doczanego do dziaajcej wirtualnej maszyny
 *  https://www.elastic.co/guide/en/apm/agent/java/current/faq.html#faq-how-does-it-work
 *  https://www.elastic.co/guide/en/apm/agent/java/current/setup-attach-cli.html
 *  
 *  Profilowanie w IntelliJ
 *  https://lp.jetbrains.com/intellij-idea-profiler/
 */

public class InstrumentationExample {

    public static void printObjectSize(Object object) {
        System.out.println("Object type: " + object.getClass() + ", size: " + InstrumentationAgent.getObjectSize(object) + " bytes");
    }

    public static void main(String[] arguments) {
        String emptyString = "";
        String string = "Estimating Object Size Using Instrumentation";
        String[] stringArray = { emptyString, string, "com.baeldung" };
        String[] anotherStringArray = new String[100];
        List<String> stringList = new ArrayList<>();
        StringBuilder stringBuilder = new StringBuilder(100);
        int maxIntPrimitive = Integer.MAX_VALUE;
        int minIntPrimitive = Integer.MIN_VALUE;
        Integer maxInteger = Integer.MAX_VALUE;
        Integer minInteger = Integer.MIN_VALUE;
        long zeroLong = 0L;
        double zeroDouble = 0.0;
        boolean falseBoolean = false;
        Object object = new Object();

        class EmptyClass {
        }
        EmptyClass emptyClass = new EmptyClass();

        class StringClass {
            public String s;
        }
        StringClass stringClass = new StringClass();

        printObjectSize(emptyString);
        printObjectSize(string);
        printObjectSize(stringArray);
        printObjectSize(anotherStringArray);
        printObjectSize(stringList);
        printObjectSize(stringBuilder);
        printObjectSize(maxIntPrimitive);
        printObjectSize(minIntPrimitive);
        printObjectSize(maxInteger);
        printObjectSize(minInteger);
        printObjectSize(zeroLong);
        printObjectSize(zeroDouble);
        printObjectSize(falseBoolean);
        printObjectSize(Day.TUESDAY);
        printObjectSize(object);
        printObjectSize(emptyClass);
        printObjectSize(stringClass);
    }

    public enum Day {
        MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY, SUNDAY
    }
}
